Type: Package
Package: datawizard
Title: Easy Data Wrangling and Statistical Transformations
Version: 0.5.0
Authors@R: c(
    person("Indrajeet", "Patil", , "patilindrajeet.science@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-1995-6531", Twitter = "@patilindrajeets")),
    person("Dominique", "Makowski", , "dom.makowski@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0001-5375-9967", Twitter = "@Dom_Makowski")),
    person("Daniel", "Lüdecke", , "d.luedecke@uke.de", role = "aut",
           comment = c(ORCID = "0000-0002-8895-3206", Twitter = "@strengejacke")),
    person("Mattan S.", "Ben-Shachar", , "matanshm@post.bgu.ac.il", role = "aut",
           comment = c(ORCID = "0000-0002-4287-4801")),
    person("Brenton M.", "Wiernik", , "brenton@wiernik.org", role = "aut",
           comment = c(ORCID = "0000-0001-9560-6336", Twitter = "@bmwiernik")),
    person("Etienne", "Bacher", , "etienne.bacher@protonmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-9271-5075"))
  )
Maintainer: Indrajeet Patil <patilindrajeet.science@gmail.com>
Description: A lightweight package to easily manipulate, clean, transform,
    and prepare your data for analysis. It also forms the data wrangling
    backend for the packages in the 'easystats' ecosystem.
License: GPL (>= 3)
URL: https://easystats.github.io/datawizard/
BugReports: https://github.com/easystats/datawizard/issues
Depends: R (>= 3.5)
Imports: insight (>= 0.18.0), stats, utils
Suggests: bayestestR, boot, brms, data.table, dplyr, effectsize, gamm4,
        ggplot2, haven, httr, knitr, lme4, mediation, parameters,
        poorman, psych, readxl, readr, rio, rmarkdown, rstanarm, see,
        testthat (>= 3.0.0), tidyr, withr
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.2.1
NeedsCompilation: no
Packaged: 2022-08-07 14:17:15 UTC; indrajeetpatil
Author: Indrajeet Patil [aut, cre] (<https://orcid.org/0000-0003-1995-6531>,
    @patilindrajeets),
  Dominique Makowski [aut] (<https://orcid.org/0000-0001-5375-9967>,
    @Dom_Makowski),
  Daniel Lüdecke [aut] (<https://orcid.org/0000-0002-8895-3206>,
    @strengejacke),
  Mattan S. Ben-Shachar [aut] (<https://orcid.org/0000-0002-4287-4801>),
  Brenton M. Wiernik [aut] (<https://orcid.org/0000-0001-9560-6336>,
    @bmwiernik),
  Etienne Bacher [aut] (<https://orcid.org/0000-0002-9271-5075>)
Repository: CRAN
Date/Publication: 2022-08-07 15:40:06 UTC
