Type: Package
Package: datawizard
Title: Easy Data Wrangling and Statistical Transformations
Version: 0.6.5
Authors@R: c(
    person("Indrajeet", "Patil", , "patilindrajeet.science@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-1995-6531", Twitter = "@patilindrajeets")),
    person("Etienne", "Bacher", , "etienne.bacher@protonmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-9271-5075")),
    person("Dominique", "Makowski", , "dom.makowski@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0001-5375-9967", Twitter = "@Dom_Makowski")),
    person("Daniel", "Lüdecke", , "d.luedecke@uke.de", role = "aut",
           comment = c(ORCID = "0000-0002-8895-3206", Twitter = "@strengejacke")),
    person("Mattan S.", "Ben-Shachar", , "matanshm@post.bgu.ac.il", role = "aut",
           comment = c(ORCID = "0000-0002-4287-4801")),
    person("Brenton M.", "Wiernik", , "brenton@wiernik.org", role = "aut",
           comment = c(ORCID = "0000-0001-9560-6336", Twitter = "@bmwiernik")),
    person("Rémi", "Thériault", , "remi.theriault@mail.mcgill.ca", role = "ctb",
           comment = c(ORCID = "0000-0003-4315-6788", Twitter = "@rempsyc")),
    person("Thomas J.", "Faulkenberry", , "faulkenberry@tarleton.edu", role = "rev"),
    person("Robert", "Garrett", , "rcg4@illinois.edu", role = "rev")
  )
Maintainer: Etienne Bacher <etienne.bacher@protonmail.com>
Description: A lightweight package to assist in key steps involved in any data 
    analysis workflow: (1) wrangling the raw data to get it in the needed form, 
    (2) applying preprocessing steps and statistical transformations, and 
    (3) compute statistical summaries of data properties and distributions. 
    It is also the data wrangling backend for packages in 'easystats' ecosystem.
    References: Patil et al. (2022) <doi:10.21105/joss.04684>.
License: GPL (>= 3)
URL: https://easystats.github.io/datawizard/
BugReports: https://github.com/easystats/datawizard/issues
Depends: R (>= 3.6)
Imports: insight (>= 0.18.8), stats, utils
Suggests: bayestestR, boot, brms, data.table, dplyr (>= 1.0),
        effectsize, gamm4, ggplot2, gt, haven, htmltools, httr, knitr,
        lme4, mediation, parameters, poorman (>= 0.2.6), psych, readxl,
        readr, rio, rmarkdown, rstanarm, see, testthat (>= 3.1.0),
        tidyr, withr
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.2.3
Config/testthat/edition: 3
Config/Needs/website: rstudio/bslib, r-lib/pkgdown,
        easystats/easystatstemplate
NeedsCompilation: no
Packaged: 2022-12-14 12:54:09 UTC; indrajeetpatil
Author: Indrajeet Patil [aut] (<https://orcid.org/0000-0003-1995-6531>,
    @patilindrajeets),
  Etienne Bacher [aut, cre] (<https://orcid.org/0000-0002-9271-5075>),
  Dominique Makowski [aut] (<https://orcid.org/0000-0001-5375-9967>,
    @Dom_Makowski),
  Daniel Lüdecke [aut] (<https://orcid.org/0000-0002-8895-3206>,
    @strengejacke),
  Mattan S. Ben-Shachar [aut] (<https://orcid.org/0000-0002-4287-4801>),
  Brenton M. Wiernik [aut] (<https://orcid.org/0000-0001-9560-6336>,
    @bmwiernik),
  Rémi Thériault [ctb] (<https://orcid.org/0000-0003-4315-6788>,
    @rempsyc),
  Thomas J. Faulkenberry [rev],
  Robert Garrett [rev]
Repository: CRAN
Date/Publication: 2022-12-14 23:50:02 UTC
