# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,datawizard_crosstabs)
S3method(as.data.frame,datawizard_tables)
S3method(as.double,parameters_kurtosis)
S3method(as.double,parameters_skewness)
S3method(as.double,parameters_smoothness)
S3method(as.numeric,parameters_kurtosis)
S3method(as.numeric,parameters_skewness)
S3method(as.numeric,parameters_smoothness)
S3method(as.prop.table,datawizard_crosstab)
S3method(as.prop.table,datawizard_crosstabs)
S3method(as.table,datawizard_crosstab)
S3method(as.table,datawizard_crosstabs)
S3method(as.table,datawizard_table)
S3method(as.table,datawizard_tables)
S3method(assign_labels,character)
S3method(assign_labels,data.frame)
S3method(assign_labels,default)
S3method(assign_labels,factor)
S3method(assign_labels,numeric)
S3method(categorize,data.frame)
S3method(categorize,default)
S3method(categorize,factor)
S3method(categorize,grouped_df)
S3method(categorize,numeric)
S3method(center,AsIs)
S3method(center,Date)
S3method(center,character)
S3method(center,data.frame)
S3method(center,default)
S3method(center,factor)
S3method(center,grouped_df)
S3method(center,logical)
S3method(center,numeric)
S3method(coef_var,default)
S3method(coef_var,numeric)
S3method(convert_na_to,character)
S3method(convert_na_to,data.frame)
S3method(convert_na_to,default)
S3method(convert_na_to,factor)
S3method(convert_na_to,numeric)
S3method(convert_to_na,Date)
S3method(convert_to_na,character)
S3method(convert_to_na,data.frame)
S3method(convert_to_na,default)
S3method(convert_to_na,factor)
S3method(convert_to_na,logical)
S3method(convert_to_na,numeric)
S3method(data_arrange,default)
S3method(data_arrange,grouped_df)
S3method(data_duplicated,data.frame)
S3method(data_duplicated,grouped_df)
S3method(data_extract,data.frame)
S3method(data_filter,data.frame)
S3method(data_filter,grouped_df)
S3method(data_merge,data.frame)
S3method(data_merge,list)
S3method(data_modify,data.frame)
S3method(data_modify,default)
S3method(data_modify,grouped_df)
S3method(data_peek,data.frame)
S3method(data_summary,data.frame)
S3method(data_summary,default)
S3method(data_summary,grouped_df)
S3method(data_summary,matrix)
S3method(data_tabulate,data.frame)
S3method(data_tabulate,default)
S3method(data_tabulate,grouped_df)
S3method(data_unique,data.frame)
S3method(data_unique,grouped_df)
S3method(describe_distribution,character)
S3method(describe_distribution,data.frame)
S3method(describe_distribution,default)
S3method(describe_distribution,factor)
S3method(describe_distribution,grouped_df)
S3method(describe_distribution,list)
S3method(describe_distribution,numeric)
S3method(display,data_codebook)
S3method(display,datawizard_crosstab)
S3method(display,datawizard_crosstabs)
S3method(display,datawizard_table)
S3method(display,datawizard_tables)
S3method(display,parameters_distribution)
S3method(format,data_codebook)
S3method(format,datawizard_crosstab)
S3method(format,datawizard_table)
S3method(format,dw_data_peek)
S3method(format,dw_groupmeans)
S3method(format,parameters_distribution)
S3method(kurtosis,data.frame)
S3method(kurtosis,default)
S3method(kurtosis,matrix)
S3method(kurtosis,numeric)
S3method(labels_to_levels,data.frame)
S3method(labels_to_levels,default)
S3method(labels_to_levels,factor)
S3method(makepredictcall,dw_transformer)
S3method(means_by_group,data.frame)
S3method(means_by_group,default)
S3method(means_by_group,numeric)
S3method(normalize,data.frame)
S3method(normalize,factor)
S3method(normalize,grouped_df)
S3method(normalize,matrix)
S3method(normalize,numeric)
S3method(plot,parameters_distribution)
S3method(plot,visualisation_recipe)
S3method(print,data_codebook)
S3method(print,data_seek)
S3method(print,datawizard_crosstab)
S3method(print,datawizard_crosstabs)
S3method(print,datawizard_table)
S3method(print,datawizard_tables)
S3method(print,dw_data_peek)
S3method(print,dw_data_summary)
S3method(print,dw_groupmeans)
S3method(print,dw_groupmeans_list)
S3method(print,dw_transformer)
S3method(print,parameters_distribution)
S3method(print,parameters_kurtosis)
S3method(print,parameters_skewness)
S3method(print,visualisation_recipe)
S3method(print_html,data_codebook)
S3method(print_html,datawizard_crosstab)
S3method(print_html,datawizard_crosstabs)
S3method(print_html,datawizard_table)
S3method(print_html,datawizard_tables)
S3method(print_html,dw_data_peek)
S3method(print_html,parameters_distribution)
S3method(print_md,data_codebook)
S3method(print_md,datawizard_crosstab)
S3method(print_md,datawizard_crosstabs)
S3method(print_md,datawizard_table)
S3method(print_md,datawizard_tables)
S3method(print_md,dw_data_peek)
S3method(print_md,parameters_distribution)
S3method(ranktransform,data.frame)
S3method(ranktransform,factor)
S3method(ranktransform,grouped_df)
S3method(ranktransform,numeric)
S3method(recode_values,character)
S3method(recode_values,data.frame)
S3method(recode_values,default)
S3method(recode_values,factor)
S3method(recode_values,numeric)
S3method(replace_nan_inf,data.frame)
S3method(replace_nan_inf,default)
S3method(rescale,data.frame)
S3method(rescale,default)
S3method(rescale,grouped_df)
S3method(rescale,numeric)
S3method(reverse,data.frame)
S3method(reverse,default)
S3method(reverse,factor)
S3method(reverse,grouped_df)
S3method(reverse,numeric)
S3method(rowid_as_column,default)
S3method(rowid_as_column,grouped_df)
S3method(skewness,data.frame)
S3method(skewness,default)
S3method(skewness,matrix)
S3method(skewness,numeric)
S3method(slide,data.frame)
S3method(slide,default)
S3method(slide,numeric)
S3method(smoothness,data.frame)
S3method(smoothness,default)
S3method(smoothness,numeric)
S3method(standardize,AsIs)
S3method(standardize,Date)
S3method(standardize,Surv)
S3method(standardize,bcplm)
S3method(standardize,biglm)
S3method(standardize,brmsfit)
S3method(standardize,character)
S3method(standardize,clm2)
S3method(standardize,data.frame)
S3method(standardize,datagrid)
S3method(standardize,default)
S3method(standardize,double)
S3method(standardize,factor)
S3method(standardize,fixest)
S3method(standardize,grouped_df)
S3method(standardize,integer)
S3method(standardize,logical)
S3method(standardize,matrix)
S3method(standardize,mediate)
S3method(standardize,mixor)
S3method(standardize,numeric)
S3method(standardize,visualisation_matrix)
S3method(standardize,wbgee)
S3method(standardize,wbm)
S3method(summary,parameters_kurtosis)
S3method(summary,parameters_skewness)
S3method(to_factor,Date)
S3method(to_factor,character)
S3method(to_factor,data.frame)
S3method(to_factor,default)
S3method(to_factor,double)
S3method(to_factor,factor)
S3method(to_factor,haven_labelled)
S3method(to_factor,logical)
S3method(to_factor,numeric)
S3method(to_numeric,Date)
S3method(to_numeric,POSIXct)
S3method(to_numeric,POSIXlt)
S3method(to_numeric,POSIXt)
S3method(to_numeric,character)
S3method(to_numeric,data.frame)
S3method(to_numeric,default)
S3method(to_numeric,double)
S3method(to_numeric,factor)
S3method(to_numeric,haven_labelled)
S3method(to_numeric,logical)
S3method(to_numeric,numeric)
S3method(unnormalize,data.frame)
S3method(unnormalize,default)
S3method(unnormalize,grouped_df)
S3method(unnormalize,numeric)
S3method(unstandardize,array)
S3method(unstandardize,character)
S3method(unstandardize,data.frame)
S3method(unstandardize,datagrid)
S3method(unstandardize,factor)
S3method(unstandardize,grouped_df)
S3method(unstandardize,matrix)
S3method(unstandardize,numeric)
S3method(unstandardize,visualisation_matrix)
S3method(winsorize,character)
S3method(winsorize,data.frame)
S3method(winsorize,factor)
S3method(winsorize,logical)
S3method(winsorize,numeric)
export(adjust)
export(as.prop.table)
export(assign_labels)
export(categorize)
export(center)
export(centre)
export(change_scale)
export(coef_var)
export(coerce_to_numeric)
export(colnames_to_row)
export(column_as_rownames)
export(contr.deviation)
export(convert_na_to)
export(convert_to_na)
export(data_addprefix)
export(data_addsuffix)
export(data_adjust)
export(data_arrange)
export(data_codebook)
export(data_duplicated)
export(data_extract)
export(data_filter)
export(data_group)
export(data_join)
export(data_match)
export(data_merge)
export(data_modify)
export(data_partition)
export(data_peek)
export(data_read)
export(data_relocate)
export(data_remove)
export(data_rename)
export(data_rename_rows)
export(data_reorder)
export(data_replicate)
export(data_restoretype)
export(data_rotate)
export(data_seek)
export(data_select)
export(data_separate)
export(data_summary)
export(data_tabulate)
export(data_to_long)
export(data_to_wide)
export(data_transpose)
export(data_ungroup)
export(data_unique)
export(data_unite)
export(data_write)
export(degroup)
export(demean)
export(describe_distribution)
export(detrend)
export(display)
export(distribution_coef_var)
export(distribution_mode)
export(empty_columns)
export(empty_rows)
export(extract_column_names)
export(find_columns)
export(kurtosis)
export(labels_to_levels)
export(mean_sd)
export(means_by_group)
export(median_mad)
export(normalize)
export(print_html)
export(print_md)
export(ranktransform)
export(recode_into)
export(recode_values)
export(remove_empty)
export(remove_empty_columns)
export(remove_empty_rows)
export(replace_nan_inf)
export(rescale)
export(rescale_weights)
export(reshape_ci)
export(reshape_longer)
export(reshape_wider)
export(reverse)
export(reverse_scale)
export(row_count)
export(row_means)
export(row_sums)
export(row_to_colnames)
export(rowid_as_column)
export(rownames_as_column)
export(skewness)
export(slide)
export(smoothness)
export(standardise)
export(standardize)
export(text_concatenate)
export(text_format)
export(text_fullstop)
export(text_lastchar)
export(text_paste)
export(text_remove)
export(text_wrap)
export(to_factor)
export(to_numeric)
export(unnormalize)
export(unstandardise)
export(unstandardize)
export(visualisation_recipe)
export(weighted_mad)
export(weighted_mean)
export(weighted_median)
export(weighted_sd)
export(winsorize)
importFrom(insight,display)
importFrom(insight,print_html)
importFrom(insight,print_md)
importFrom(stats,makepredictcall)
