#' @docType data
#' @title Sample dataset from the National Health and Nutrition Examination Survey
#' @name nhanes_sample
#' @keywords data
#'
#' @description Selected variables from the National Health and Nutrition Examination
#'              Survey that are used in the example from Lumley (2010), Appendix E.
#'
#' @references Lumley T (2010). Complex Surveys: a guide to analysis using R. Wiley
NULL
