#' @docType data
#' @title Sample dataset from the National Health and Nutrition Examination Survey
#' @name nhanes_sample
#' @keywords data
#'
#' @description Selected variables from the National Health and Nutrition Examination
#'              Survey that are used in the example from Lumley (2010), Appendix E.
#'
#' @references Lumley T (2010). Complex Surveys: a guide to analysis using R. Wiley
NULL


#' @docType data
#' @title Sample dataset from the EFC Survey
#' @name efc
#' @keywords data
#'
#' @description Selected variables from the EUROFAMCARE survey. Useful when
#' testing on "real-life" data sets, including random missing values. This
#' data set also has value and variable label attributes.
NULL
