---
title: "Overview of Vignettes"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Overview of Vignettes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r message=FALSE, warning=FALSE, include=FALSE}
library(knitr)
knitr::opts_chunk$set(
  echo = TRUE,
  collapse = TRUE,
  warning = FALSE,
  message = FALSE,
  comment = "#>",
  eval = TRUE
)
```

All package vignettes are available at [https://easystats.github.io/datawizard/](https://easystats.github.io/datawizard/).

## Function Overview

* [Function Reference](https://easystats.github.io/datawizard/reference/index.html)


## Data Preparation

* [Coming from 'tidyverse'](https://easystats.github.io/datawizard/articles/tidyverse_translation.html)
* [A quick summary of selection syntax in `{datawizard}`](https://easystats.github.io/datawizard/articles/selection_syntax.html)


## Statistical Transformations

* [Data Standardization](https://easystats.github.io/datawizard/articles/standardize_data.html)
