# data_to_long works - complex dataset

    Code
      str(long)
    Output
      'data.frame':	70000 obs. of  6 variables:
       $ gender     : int  1 1 1 1 1 1 1 1 1 1 ...
       $ education  : int  NA NA NA NA NA NA NA NA NA NA ...
       $ age        : int  16 16 16 16 16 16 16 16 16 16 ...
       $ Participant: num  61617 61617 61617 61617 61617 ...
       $ Item       : chr  "A1" "A2" "A3" "A4" ...
       $ Score      : int  2 4 3 4 4 2 3 3 4 4 ...

